@@ -1,4 +1,4 @@
-from aio_statsd.client import DogStatsdClient, GraphiteClient, StatsdClient, TelegrafStatsdClient
+from aio_statsd.client import DogStatsdClient, GraphiteClient, StatsdClient, TelegrafClient, TelegrafStatsdClient
 from aio_statsd.protocol import DogStatsdProtocol, StatsdProtocol, TelegrafStatsdProtocol
 from aio_statsd.transport_layer_protocol import ProtocolFlag
 
@@ -9,6 +9,7 @@ __all__ = [
     "ProtocolFlag",
     "StatsdClient",
     "StatsdProtocol",
+    "TelegrafClient",
     "TelegrafStatsdProtocol",
     "TelegrafStatsdClient",
 ]
