@@ -24,97 +24,88 @@ class Client:
         protocol: ProtocolFlag = ProtocolFlag.udp,
         timeout: int = 0,
         debug: bool = False,
-        close_timeout: int = 5,
-        create_timeout: int = 5,
+        close_timeout: int = 9,
+        create_timeout: int = 9,
         max_len: int = 1000,
-    ):
+    ) -> None:
         self._queue_empty: "object" = object()
         self._max_len: int = max_len
         self._deque: Deque[str] = deque(maxlen=self._max_len)
-        self._listen_future: Optional[asyncio.Future] = None
+        self._listen_future: asyncio.Future = asyncio.Future()
+        self._listen_future.set_result(True)
 
-        self.is_closed: bool = True
         self._close_timeout: int = close_timeout
-        self._conn_config_dict: Dict[str, Any] = {
-            "host": host,
-            "port": port,
-            "protocol_flag": protocol,
-            "debug": debug,
-            "timeout": timeout,
-            "create_timeout": create_timeout,
-        }
 
-        self.connection: Connection = Connection(**self._conn_config_dict)
+        self.connection: Connection = Connection(
+            host=host,
+            port=port,
+            protocol_flag=protocol,
+            debug=debug,
+            timeout=timeout,
+            create_timeout=create_timeout,
+        )
 
     async def __aenter__(self) -> "Client":
         await self.connect()
         return self
 
-    async def __aexit__(self, *args) -> None:
-        async def await_deque_empty():
-            while True:
-                if not self._deque:
-                    break
-                await asyncio.sleep(0.1)
-
-        try:
-            await asyncio.wait_for(await_deque_empty(), 9)
-        except asyncio.TimeoutError:
-            pass
+    async def __aexit__(self, *args: Any) -> None:
         await self.close()
 
     async def connect(self) -> None:
         if not self.is_closed:
-            raise ConnectionError(f"aiostatsd client already connected")
+            raise ConnectionError(f"{self.__class__.__name__} already connected")
         await self.connection.connect()
         self._deque = deque(maxlen=self._max_len)
-        self.is_closed = False
         self._listen_future = asyncio.ensure_future(self._listen())
-        logging.info(f"create aiostatsd client{self}")
+        logging.info(f"create {self.__class__.__name__}")
+
+    @property
+    def is_closed(self) -> bool:
+        return self._listen_future.done()
 
     async def close(self) -> None:
-        self.is_closed = True
-        if self._listen_future:
-            await self._listen_future
-            self._listen_future = None
-
-    async def _close(self):
-        self.is_closed = True
-
-        async def before_close():
-            while True:
-                value: Optional[str] = self._get_by_queue()
-                if value:
-                    await self._real_send(value)
-                else:
-                    break
+        if not self.connection.is_closed:
+            async def wait_send_msg() -> None:
+                while len(self._deque) > 0:
+                    await asyncio.sleep(0.1)
+            try:
+                await asyncio.wait_for(wait_send_msg(), timeout=self._close_timeout)
+            except asyncio.TimeoutError:
+                pass
+            await self.connection.await_close()
 
+        self._deque.clear()
+        if not self._listen_future.cancelled():
+            self._listen_future.cancel()
         try:
-            await asyncio.wait_for(before_close(), timeout=self._close_timeout)
-        except asyncio.TimeoutError:
+            await self._listen_future
+        except asyncio.CancelledError:
             pass
-        await self.connection.await_close()
 
     def _get_by_queue(self) -> Optional[str]:
         try:
             if self._deque:
                 return self._deque.pop()
-            return None
         except IndexError:
-            return None
+            pass
+        return None
 
     async def _listen(self) -> None:
         try:
-            while not self.is_closed:
+            while not self.connection.is_closed:
                 value: Optional[str] = self._get_by_queue()
                 if value:
                     await self._real_send(value)
                 else:
-                    await asyncio.sleep(0.01)
+                    await asyncio.sleep(0.05)
+
+                if self.connection.future.done():
+                    await self.connection.future
+        except asyncio.CancelledError:
+            pass
         except Exception as e:
-            logging.error(f"aiostatsd listen status:{self.is_closed} error: {e}")
-        finally:
-            await self._close()
+            logging.exception(f"status:{self.is_closed} error: {e}")
 
     async def _real_send(self, msg: str) -> None:
         try:
@@ -128,11 +119,13 @@ class Client:
                 logging.error(f"send msd error:{e}, drop msg:{msg}")
 
     def send(self, msg: str) -> None:
+        if self.is_closed:
+            raise ConnectionError(f"{self.__class__.__name__} already close.")
         try:
             # if queue full, auto del last value(queue[-1])
             self._deque.appendleft(msg)
         except Exception as e:
-            logging.error(f"aiostatsd put:{msg} to queue error:{e}")
+            logging.error(f"put:{msg} to queue error:{e}")
 
 
 class GraphiteClient(Client):
@@ -158,8 +151,10 @@ class GraphiteClient(Client):
             max_len=max_len,
         )
 
-    def send_graphite(self, key: str, value: int = 0, timestamp: int = int(time.time()), interval: int = 10) -> None:
+    def send_graphite(self, key: str, value: int = 0, timestamp: Optional[int] = None, interval: int = 10) -> None:
         """interval: Multiple clients timestamp interval synchronization"""
+        if not timestamp:
+            timestamp = int(time.time())
         timestamp = int(timestamp) // interval * interval
         msg: str = "{} {} {}".format(key, value, timestamp)
         self.send(msg)
@@ -207,12 +202,12 @@ class TelegrafClient(Client):
         return value.replace(",", "\\,").replace(" ", "\\ ").replace("=", "\\=").replace('"', "\\")
 
     def send_telegraf(
-            self,
-            key: str,
-            field_dict: Dict[str, Any],
-            tag_dict: Optional[Dict[str, str]] = None,
-            user_server_time: Optional[bool] = None,
-            enable_replace_special: Optional[bool] = None,
+        self,
+        key: str,
+        field_dict: Dict[str, Any],
+        tag_dict: Optional[Dict[str, str]] = None,
+        user_server_time: Optional[bool] = None,
+        enable_replace_special: Optional[bool] = None,
     ) -> None:
         if enable_replace_special is None:
             enable_replace_special = self._enable_replace_special
@@ -245,10 +240,7 @@ class TelegrafClient(Client):
             )
         else:
             field_str = " " + ",".join(
-                [
-                    f"{key}={self._replace_field_value(value)}"
-                    for key, value in field_dict.items()
-                ]
+                [f"{key}={self._replace_field_value(value)}" for key, value in field_dict.items()]
             )
 
         msg: str = f"{key}{tag_str}{field_str}"
@@ -271,6 +263,8 @@ class StatsdClient(Client):
         create_timeout: int = 5,
         max_len: int = 10000,
     ) -> None:
+        if protocol == ProtocolFlag.tcp:
+            raise ValueError(f"{self.__class__.__name__} not support {protocol.value} protocol")
         super().__init__(
             host=host,
             port=port,
@@ -315,6 +309,14 @@ class StatsdClient(Client):
         statsd_protocol: "StatsdProtocol" = StatsdProtocol().sets(key, value)
         self.send_statsd(statsd_protocol, sample_rate)
 
+    def increment(self, key: str, value: int, sample_rate: Union[int, float, None] = None) -> None:
+        statsd_protocol: "StatsdProtocol" = StatsdProtocol().increment(key, value)
+        self.send_statsd(statsd_protocol, sample_rate)
+
+    def decrement(self, key: str, value: int, sample_rate: Union[int, float, None] = None) -> None:
+        statsd_protocol: "StatsdProtocol" = StatsdProtocol().decrement(key, value)
+        self.send_statsd(statsd_protocol, sample_rate)
+
     @contextmanager
     def timeit(self, key: str, sample_rate: Union[int, float, None] = None) -> Iterator[None]:
         """
