@@ -23,23 +23,30 @@ class Connection(object):
         self._debug: bool = debug
         self._protocol_flag: ProtocolFlag = protocol_flag
         self._create_timeout: int = create_timeout
-
+        self._host: str = host
+        self._port: int = port
         self._connection_info = f"{protocol_flag}://{host}:{port}"
-        _loop = get_event_loop()
         if protocol_flag == ProtocolFlag.udp:
             self._connection: Union[DatagramProtocol, TcpProtocol] = DatagramProtocol(timeout=timeout)
-            self._connection_proxy = _loop.create_datagram_endpoint(lambda: self._connection, remote_addr=(host, port))
         elif protocol_flag == ProtocolFlag.tcp:
             self._connection = TcpProtocol(timeout=timeout)
-            self._connection_proxy = _loop.create_connection(lambda: self._connection, host=host, port=port)
         else:
             raise ConnectionError(f"Not support protocol:{protocol_flag}")
 
         self.sendto = self._sendto if not debug else self._sendto_debug
 
     async def connect(self) -> None:
+        _loop = get_event_loop()
+        if self._protocol_flag == ProtocolFlag.udp:
+            connection_proxy = _loop.create_datagram_endpoint(
+                lambda: self._connection, remote_addr=(self._host, self._port)
+            )
+        elif self._protocol_flag == ProtocolFlag.tcp:
+            connection_proxy = _loop.create_connection(lambda: self._connection, host=self._host, port=self._port)
+        else:
+            raise ConnectionError(f"Not support protocol:{self._protocol_flag}")
         try:
-            await asyncio.wait_for(self._connection_proxy, timeout=self._create_timeout)
+            await asyncio.wait_for(connection_proxy, timeout=self._create_timeout)
         except asyncio.TimeoutError as e:
             raise TimeoutError(f"create connection:{self._connection_info} timeout") from e
         logging.debug(f"create connection:{self._connection_info}")
@@ -57,12 +64,16 @@ class Connection(object):
     def is_closed(self) -> bool:
         return self._connection.is_closed
 
-    def close(self):
+    @property
+    def future(self) -> asyncio.Future:
+        return self._connection.future
+
+    def close(self) -> None:
         self._connection.close()
 
-    async def wait_closed(self):
+    async def wait_closed(self) -> None:
         await self._connection.wait_closed()
 
-    async def await_close(self):
+    async def await_close(self) -> None:
         await self._connection.await_close()
         logging.debug(f"close connection: {self._connection_info}")
